<?php

namespace Drupal\storychief\Plugin\FieldHandlerType;


/**
 * Base class to handle taxonomy terms.
 *
 * @package Drupal\storychief\Plugin\StorychiefFieldHandler
 */
class TaxonomyTermFieldHandlerType extends EntityReferenceFieldHandlerType
{

  public function getValue()
  {
    $terms = [];
    $term_names = parent::getValue();

    foreach ($term_names as $term_name) {
      $term = $this->loadTaxonomyTerm($term_name);

      if (!$term && $this->getFieldDefinition()->getSetting('handler_settings')['auto_create']) {
        $term = $this->createTaxonomyTerm($term_name);
      }

      array_push($terms, $term);
    }

    return $terms;
  }

  protected function loadTaxonomyTerm($term_name)
  {
    $vocabularies = $this->getFieldDefinition()->getSetting('handler_settings')['target_bundles'];

    $terms = $this->entityTypeManager
      ->getStorage('taxonomy_term')
      ->loadByProperties([
        'vid' => array_values($vocabularies),
        'name' => $term_name,
        'langcode' => $this->getEntity()->language()->getId(),
      ]);

    // A term was found
    if (empty($terms)) {
      return null;
    }

    return array_values($terms)[0];
  }

  protected function createTaxonomyTerm($term_name)
  {
    $vocabularies = $this->getFieldDefinition()->getSetting('handler_settings')['target_bundles'];
    $term = $this->entityTypeManager
      ->getStorage('taxonomy_term')
      ->create([
        'name' => $term_name,
        'vid' => array_values($vocabularies)[0],
        'langcode' => $this->getEntity()->language()->getId(),
      ]);

    $term->save();

    return $term;
  }
}
